@@ -5,12 +5,15 @@ from abc import ABC, abstractmethod
 from itertools import chain
 from math import inf
 from numbers import Real
-from typing import Iterable, Tuple, Dict, List, TypeVar, Generic, Any, Sequence
+from typing import Iterable, Tuple, Dict, List, TypeVar, Generic, Any, Sequence, Type
 
 from smallgraphlib.basic_graphs import Graph, DirectedGraph
-from smallgraphlib.core import Node, Edge, AbstractGraph, InvalidGraphAttribute
+from smallgraphlib.core import AbstractGraph, InvalidGraphAttribute
+from smallgraphlib.custom_types import Node, Edge
 from smallgraphlib.utilities import cached_property
 
+_AbstractLabeledGraph = TypeVar("_AbstractLabeledGraph", bound="AbstractLabeledGraph")
+_AbstractWeightedGraph = TypeVar("_AbstractWeightedGraph", bound="AbstractWeightedGraph")
 Label = TypeVar("Label")
 LabeledEdge = Tuple[Node, Node, Label]
 WeightedEdge = Tuple[Node, Node, float]
@@ -39,11 +42,26 @@ class AbstractLabeledGraph(AbstractGraph, ABC, Generic[Label]):
         )
 
     def __repr__(self):
-        labeled_edges = (repr((*edge, label)) for edge, label in self._labels.items())
+        labeled_edges = (repr(labeled_edge) for labeled_edge in self.labeled_edges)
         return f"{self.__class__.__name__}({tuple(self.nodes)!r}, {', '.join(labeled_edges)})"
 
+    @cached_property
+    def labeled_edges(self) -> tuple[LabeledEdge, ...]:
+        return tuple(  # type: ignore
+            sorted(
+                (
+                    *edge,  # type: ignore
+                    label,
+                )
+                for edge, labels in self._labels.items()
+                for label in labels
+            )
+        )
+
     @classmethod
-    def from_dict(cls, edge_label_dict: dict = None, /, **edge_label):
+    def from_dict(
+        cls: Type[_AbstractLabeledGraph], edge_label_dict: dict = None, /, **edge_label
+    ) -> _AbstractLabeledGraph:
         """Construct a directed graph using a {edge_name: label} dictionnary (or keywords).
 
         All edges' names must be two letters strings (like "AB"), each letter representing a node.
@@ -63,12 +81,12 @@ class AbstractLabeledGraph(AbstractGraph, ABC, Generic[Label]):
         return cls(nodes, *((*edge, label) for edge, label in edge_label_dict.items()))  # type: ignore
 
     @classmethod
-    def from_string(cls, string: str):
+    def from_string(cls: Type[_AbstractLabeledGraph], string: str) -> _AbstractLabeledGraph:
         """LabeledGraph.from_string("A:B=label,C='other label' B:C=5 C D:C")
-        will generate a graph of 3 nodes, A, B and C, with edges A->B, A->C
-        and B->C and respective labels 'label', 'other label' and 5.
+        will generate a graph of 4 nodes, A, B, C and D, with edges A->B, A->C, B->C and C->D
+        and respective labels 'label', 'other label', 5 and `None`.
 
-        Note that labels containing a space must be surrounded by quotes.
+        Note that labels containing a space must be surrounded by single or double quotes.
         Labels containing only digits will be converted to numbers (integers or floats),
         except if surrounded by quotes.
 
@@ -177,8 +195,25 @@ class AbstractWeightedGraph(AbstractLabeledGraph, ABC):
         """
         if node1 == node2:
             return 0
-        values = self.weights[self._edge(node1, node2)]
-        return aggregator(values) if values else default
+        values = self.weights.get(self._edge(node1, node2), [default])
+        return aggregator(values)
+
+    def _tikz_labels(self, node1: Node, node2: Node) -> list[str]:
+        """Overwrite this method to modify tikz value for some labels."""
+        labels = self._labels.get(self._edge(node1, node2), [])
+
+        def format_(label):
+            # Note: math.isinf() supports `sympy.oo` too.
+            if label is None:
+                return ""
+            elif math.isinf(label) and label > 0:
+                return r"$\infty$"
+            elif math.isinf(label) and label < 0:
+                return r"$-\infty$"
+            else:
+                return str(label)
+
+        return [format_(label) for label in labels]
 
     @cached_property
     def total_weight(self) -> float:
@@ -194,8 +229,10 @@ class AbstractWeightedGraph(AbstractLabeledGraph, ABC):
 
     @classmethod
     def from_matrix(
-        cls, matrix: Iterable[Iterable[float]], nodes_names: Iterable[Node] = None
-    ) -> "AbstractGraph":
+        cls: Type[_AbstractWeightedGraph],
+        matrix: Iterable[Iterable[float]],
+        nodes_names: Iterable[Node] = None,
+    ) -> _AbstractWeightedGraph:
         """Construct the graph corresponding to the given adjacency matrix.
 
         Matrix must be a matrix of positive real numbers
@@ -235,17 +272,19 @@ class WeightedGraph(AbstractWeightedGraph, LabeledGraph):
                             "The diagonal coefficients of the weights matrix must be nul, "
                             f"but matrix[{i}][{i}]={weight}."
                         )
-
+                    # Don't append loops.
                 else:
                     if matrix[i][j] != matrix[j][i]:
                         raise ValueError(
                             "The adjacency matrix of an undirected graph must be symmetric, "
                             f"but matrix[{i}][{j}]={matrix[i][j]} != matrix[{j}][{i}]={matrix[j][i]}"
                         )
-                edges.append((i + 1, j + 1, weight))
+                    # If weight is infinite, nodes are not adjacent.
+                    if not math.isinf(weight):
+                        edges.append((i + 1, j + 1, weight))
         return edges
 
-    def minimum_spanning_tree(self) -> Graph:
+    def minimum_spanning_tree(self) -> "WeightedGraph":
         """Use Prim's algorithm to return a minimum weight spanning tree.
 
         A spanning tree of a graph G is a subgraph of G who is a tree and contains all the nodes of G.
@@ -313,5 +352,8 @@ class WeightedDirectedGraph(AbstractWeightedGraph, LabeledDirectedGraph):
                             "The diagonal coefficients of the weights matrix must be nul, "
                             f"but matrix[{i}][{i}]={weight}."
                         )
-                edges.append((i + 1, j + 1, weight))
+
+                elif not math.isinf(weight):
+                    # If weight is infinite, nodes are not adjacent.
+                    edges.append((i + 1, j + 1, weight))
         return edges
